package org.eclipse.jdt.internal.compiler.apt.model;

import java.util.Iterator;
import org.eclipse.jdt.internal.compiler.util.Util;
import java.util.Collection;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.element.VariableElement;
import javax.lang.model.element.AnnotationValueVisitor;
import org.eclipse.jdt.internal.compiler.problem.ShouldNotImplement;
import org.eclipse.jdt.internal.compiler.impl.FloatConstant;
import org.eclipse.jdt.internal.compiler.impl.DoubleConstant;
import org.eclipse.jdt.internal.compiler.impl.LongConstant;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseProcessingEnvImpl;
import org.eclipse.jdt.internal.compiler.lookup.TypeIds;
import javax.lang.model.element.AnnotationValue;

public class AnnotationValueImpl implements AnnotationValue, TypeIds
{
    private static final int T_AnnotationMirror = -1;
    private static final int T_EnumConstant = -2;
    private static final int T_ClassObject = -3;
    private static final int T_ArrayType = -4;
    private final BaseProcessingEnvImpl _env;
    private final Object _value;
    private final int _kind;
    
    public AnnotationValueImpl(BaseProcessingEnvImpl env, Object value, TypeBinding type) {
        this._env = env;
        int[] kind = { 0 };
        if (type == null) {
            this._value = this.convertToMirrorType(value, type, kind);
            this._kind = kind[0];
        }
        else if (type.isArrayType()) {
            List<AnnotationValue> convertedValues = null;
            TypeBinding valueType = ((ArrayBinding)type).elementsType();
            if ((value instanceof Object[])) {
                Object[] values = (Object[])value;
                convertedValues = new ArrayList(values.length);
                for (Object oneValue : values) {
                    convertedValues.add(new AnnotationValueImpl(this._env, oneValue, valueType));
                }
            }
            else {
                convertedValues = new ArrayList(1);
                convertedValues.add(new AnnotationValueImpl(this._env, value, valueType));
            }
            this._value = Collections.unmodifiableList(convertedValues);
            this._kind = -4;
        }
        else {
            this._value = this.convertToMirrorType(value, type, kind);
            this._kind = kind[0];
        }
    }
    
    private Object convertToMirrorType(Object value, TypeBinding type, int[] kind) {
        if (type == null) {
            kind[0] = 11;
            return "<error>";
        }
        if (type instanceof BaseTypeBinding || type.id == 11) {
            if (value == null) {
                if (type instanceof BaseTypeBinding || type.id == 11) {
                    kind[0] = 11;
                    return "<error>";
                }
                if (type.isAnnotationType()) {
                    kind[0] = -1;
                    return this._env.getFactory().newAnnotationMirror(null);
                }
            }
            else if ((value instanceof Constant)) {
                if ((type instanceof BaseTypeBinding)) {
                    kind[0] = ((BaseTypeBinding)type).id;
                }
                else if (type.id == 11) {
                    kind[0] = ((Constant)value).typeID();
                }
                else {
                    kind[0] = 11;
                    return "<error>";
                }
                switch (kind[0]) {
                    case 5: {
                        return Boolean.valueOf(((Constant)value).booleanValue());
                    }
                    case 3: {
                        return Byte.valueOf(((Constant)value).byteValue());
                    }
                    case 2: {
                        return Character.valueOf(((Constant)value).charValue());
                    }
                    case 8: {
                        return Double.valueOf(((Constant)value).doubleValue());
                    }
                    case 9: {
                        return Float.valueOf(((Constant)value).floatValue());
                    }
                    case 10: {
                        try {
                            if (value instanceof LongConstant || value instanceof DoubleConstant || value instanceof FloatConstant) {
                                kind[0] = 11;
                                return "<error>";
                            }
                            return Integer.valueOf(((Constant)value).intValue());
                        }
                        catch (ShouldNotImplement shouldNotImplement) {
                            kind[0] = 11;
                            return "<error>";
                        }
                    }
                    case 11: {
                        return ((Constant)value).stringValue();
                    }
                    case 7: {
                        return Long.valueOf(((Constant)value).longValue());
                    }
                    case 4: {
                        return Short.valueOf(((Constant)value).shortValue());
                    }
                    default: {
                        break;
                    }
                }
            }
        }
        else if (type.isEnum()) {
            if ((value instanceof FieldBinding)) {
                kind[0] = -2;
                return this._env.getFactory().newElement((FieldBinding)value);
            }
            kind[0] = 11;
            return "<error>";
        }
        else if (type.isAnnotationType()) {
            if ((value instanceof AnnotationBinding)) {
                kind[0] = -1;
                return this._env.getFactory().newAnnotationMirror((AnnotationBinding)value);
            }
        }
        else if ((value instanceof TypeBinding)) {
            kind[0] = -3;
            return this._env.getFactory().newTypeMirror((TypeBinding)value);
        }
        kind[0] = 11;
        return "<error>";
    }
    
    @Override
    public <R, P> R accept(AnnotationValueVisitor<R, P> v, P p) {
        switch (this._kind) {
            case 5: {
                return v.visitBoolean((boolean)this._value, p);
            }
            case 3: {
                return v.visitByte((byte)this._value, p);
            }
            case 2: {
                return v.visitChar((char)this._value, p);
            }
            case 8: {
                return v.visitDouble((double)this._value, p);
            }
            case 9: {
                return v.visitFloat((float)this._value, p);
            }
            case 10: {
                return v.visitInt((int)this._value, p);
            }
            case 11: {
                return v.visitString((String)this._value, p);
            }
            case 7: {
                return v.visitLong((long)this._value, p);
            }
            case 4: {
                return v.visitShort((short)this._value, p);
            }
            case -2: {
                return v.visitEnumConstant((VariableElement)this._value, p);
            }
            case -3: {
                return v.visitType((TypeMirror)this._value, p);
            }
            case -1: {
                return v.visitAnnotation((AnnotationMirror)this._value, p);
            }
            case -4: {
                return v.visitArray((List)this._value, p);
            }
            default: {
                return null;
            }
        }
    }
    
    @Override
    public Object getValue() {
        return this._value;
    }
    
    @Override
    public boolean equals(Object obj) {
        return obj instanceof AnnotationValueImpl && this._value.equals(((AnnotationValueImpl)obj)._value);
    }
    
    @Override
    public int hashCode() {
        return this._value.hashCode() + this._kind;
    }
    
    @Override
    public String toString() {
        if (this._value == null) {
            return "null";
        }
        if ((this._value instanceof String)) {
            String value = (String)this._value;
            StringBuffer sb3 = new StringBuffer();
            sb3.append('\"');
            for (int i = 0; i < value.length(); ++i) {
                Util.appendEscapedChar(sb3, value.charAt(i), true);
            }
            sb3.append('\"');
            return sb3.toString();
        }
        if ((this._value instanceof Character)) {
            StringBuffer sb2 = new StringBuffer();
            sb2.append('\'');
            Util.appendEscapedChar(sb2, (char)this._value, false);
            sb2.append('\'');
            return sb2.toString();
        }
        if ((this._value instanceof VariableElement)) {
            VariableElement enumDecl = (VariableElement)this._value;
            return enumDecl.asType().toString() + "." + enumDecl.getSimpleName();
        }
        if ((this._value instanceof Collection)) {
            Collection<AnnotationValue> values = (Collection)this._value;
            StringBuilder sb = new StringBuilder();
            sb.append('{');
            boolean first = true;
            for (AnnotationValue annoValue : values) {
                if (!first) {
                    sb.append(", ");
                }
                first = false;
                sb.append(annoValue.toString());
            }
            sb.append('}');
            return sb.toString();
        }
        if ((this._value instanceof TypeMirror)) {
            return this._value.toString() + ".class";
        }
        return this._value.toString();
    }
}
